#!/usr/bin/perl
# 
# photocomments.pl  Version 0.1
# Convert different forms of comments descriptions from 
# jpeg files. 
# (c) 2004 Gregor Maier <gregor@majordomus.org>
# Did you like the script? If so I would be glad, if you could send
# me a postcard (Gregor Maier, Unterfeldstrasse 6, A-6971 Hard, Austria)
# 
# Can read descript.ion files (acdsee) and jpeg comments
#
# can write: descript.ion files, jpeg comments, gallery-csv-files
# (needed for import to gallery)
# 
# Caveeat: Cannot handle filenames with spaces
# 
# A rather quick hack. You may want to BACKUP YOUR PHOTOS before running
# my script. I don't suppose that my script will actually damage the
# picture but the comments may be at stake.
#
#######################################################################
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
######################################################################

use strict;
use File::Basename;
use File::Temp;
use Getopt::Std;

########### Configure the Path to convert and identify here ##############
# both programs are part of the ImageMagick suite 

my $CONVERT="/usr/bin/convert";
my $IDENTIFY="/usr/bin/identify";

##########################################################################

# You shouldn't need to change anything here
my $write_desc = 0;
my $desc_fn = "descript.ion";
my $write_gallery = 0;
my $gallery_fn = "gallery.csv";
my $write_comment = 0;
my $do_bak = 0;
my $dir;


my ($desc_fh, $desc_tmp_fn);
my ($gallery_fh, $gallery_tmp_fn);

sub main {
	(-x $IDENTIFY) or die "ERROR: identify binary $IDENTIFY does not exists or is not executeable\nAdjust path in the perlscript\n";
	(-x $CONVERT) or die "ERROR: convert binary $CONVERT does not exists or is not executeable\nAdjust path in the perlscript\n";
	&parseopts;
}

# initialize a filehandle for descript.ion output
sub init_desc {
	if ($do_bak && (-f "$dir/$desc_fn") ) {
		system("cp $dir/$desc_fn $dir/$desc_fn.bak");
	}
	# We need a tmpfile if src and dest are descript.ion
	($desc_fh, $desc_tmp_fn) = tmpnam() or die "$!";
}

# initializs a filehandle for gallery.csv output
sub init_gallery {
	if ($do_bak && (-f "$dir/$gallery_fn") ) {
		system("cp $dir/$gallery_fn $dir/$gallery_fn.bak");
	}
	# gallery is not a source ==> we don't need a tmp filename
	$gallery_tmp_fn = "$dir/$gallery_fn";
	open $gallery_fh,">$gallery_tmp_fn" or die "$!";
	print $gallery_fh "Filename;Description\n";
}



sub write_it  {
	my ($pic_fn, $pic_desc) = @_;
	# TODO: Spaces im Filenamen bei $pic_fn
	if ($write_desc) {
		print $desc_fh "$pic_fn $pic_desc\n"; 
	}
	if ($write_gallery) {
		print $gallery_fh "$pic_fn;$pic_desc\n"; 
	}
	if ($write_comment) {
		system("$CONVERT -comment '$pic_desc' '$dir/$pic_fn' '$dir/$pic_fn'");
	}
}

sub from_desc {
	my ($line, $pic_fn, $pic_desc);
	open IN, "<$dir/$desc_fn" or die $!;

	while ($line=<IN>) {
		chomp($line);
		# TODO Anfuerhungszeich --> spaces im filename
		if ($line =~ /^"/) {
			print "There are spaces in the filename, skipping.\n";
		}
		else {
			($pic_fn, $pic_desc) = split " ", $line, 2;
			write_it($pic_fn, $pic_desc);
		}
	}
}  # end from_desc

# All comments / descriptions are cleared
sub from_clear {
	my (@entries, $entry);
	opendir (DIR, $dir) or die "Cannot opendir $dir: $!";
	@entries = grep { /(\.JPG)|(\.jpeg)/i && -f "$dir/$_"} readdir(DIR);
	foreach $entry (@entries) {
		#print "GMM>>$entry\n";
		write_it($entry, "");
	}
}

# comments / descriptions are taken from jpeg comments
sub from_comment {
	my (@entries, $entry,$pic_desc);
	opendir (DIR, $dir) or die "Cannot opendir $dir: $!";
	@entries = grep { /(\.JPG)|(\.jpeg)/i && -f "$dir/$_"} readdir(DIR);
	foreach $entry (@entries) {
		$pic_desc = `$IDENTIFY -format "%c" "$dir/$entry"` or print "ERROR: $dir/$entry\n";
		chomp($pic_desc);
		$pic_desc =~ s/\n/\t/gs;
		#print "$entry: >>$pic_desc<<\n";
		write_it($entry, $pic_desc);
	}
}

sub usage {
	my $exitcode = shift || 0; 
	my $base = basename($0);	
	print STDERR << "EOT";
$base can convert comments / description for jpeg photo 
galleries to differnt formats. This script will convert a whole directory
at a time.  Currently the following formats are supported:

Usage:
	$base -h       display this help message
	$base sourcespec  destsped [options] directory

sourcespec: -s comment|desc|clear
	-s comment  Use jpeg's internal comment as the source
	-s desc     Use a descript.ion file as source
	-s clear    An empty source. Clear all the comments

destspec: specity destination: -a|([-c] [-d] [-g])
	-c          Write a jped comment to each file
	-d          Write description file with filename  descript.ion
	-g          Write a csv file for gallery import filename:  gallery.csv 
	-a          Write to all possible destinations

options
	-b          Backup old descript.ion an gallery.csv files


Example:
	$base -s comment -d -g /home/gregor/pics
Will read the internal comments of all jpg files in /home/gregor/pics, 
create (or overwrite) a descript.ion and a gallery.cvs file in 
/home/gregor/pics
EOT
	exit $exitcode;
}


sub parseopts  {
	my %opts=();

	getopts("hs:cdgab", \%opts) or usage(1);
	if ($opts{h}) {
		usage(0);
	}
	if ($#ARGV<0) {
		print STDERR "ERROR: You must specify the directory to process\n\n";
		usage(1);
	}
	elsif($#ARGV>0) {
		print STDERR "ERROR: Too many arguments. Specity just one directory to process\n\n";
		usage(1);
	}
	$dir=$ARGV[0];
	if ( !-d $dir) {
		print STDERR "ERROR: $dir is not a directory\n\n";
		usage(1);
	}
	if ($opts{b}) {
		$do_bak = 1;
	}
	if ($opts{d} || $opts{a}) {
		$write_desc = 1;
		init_desc();
	}
	if ($opts{c} || $opts{a}) {
		$write_comment = 1;
	}
	if ($opts{g} || $opts{a}) {
		$write_gallery = 1;
		init_gallery();
	}
	if (!($write_desc||$write_comment||$write_gallery)) {
		print STDERR "ERROR: You must specify one ore more destinations\n\n";
		usage(1);
	}
	if ($opts{s} eq "comment") {
		from_comment;
	}
	elsif ($opts{s} eq "desc") {
		from_desc;
	}
	elsif ($opts{s} eq "clear") {
		from_clear;	
	}
	else {
		print STDERR "ERROR: You must specify the source\n\n";
		usage(1);
	}
}

END {
	close $desc_fh if defined($desc_fh);
	close $gallery_fh if defined($gallery_fh);
	system("mv $desc_tmp_fn $dir/$desc_fn") if (-f $desc_tmp_fn);
	#system("mv $gallery_tmp_fn $dir/$gallery_fn") if (-f $gallery_tmp_fn);
	system("rm -f $desc_tmp_fn") if (-f $desc_tmp_fn);
	#system("rm -f $gallery_tmp_fn") if (-f $gallery_tmp_fn);
}
&main;
